@@ -1,5 +1,5 @@
 use std::env;
-use std::io::{self, BufRead, BufReader};
+use std::io::{self, BufRead, BufReader, Stderr, Stdout, Write};
 use std::process::exit;
 use tokenizer::{Token, tokenize};
 use permutate::Permutator;
@@ -10,30 +10,8 @@ mod man;
 
 use std::fs;
 
-/// `Args` is a collection of critical options and arguments that were collected at
-/// startup of the application.
-pub struct Args {
-    /// The number of jobs to run in parallel.
-    pub ncores:     usize,
-    /// Whether stdout/stderr of each job should be handled serially or not.
-    ///
-    /// **NOTE:** This has a performance cost when enabled.
-    pub grouped:    bool,
-    /// Whether the platform's shell should be used or not.
-    ///
-    /// **NOTE:** This has a large performance cost.
-    pub uses_shell: bool,
-    /// Evaluates to true if the first argument is either `:::` or `::::`.
-    pub inputs_are_commands: bool,
-    /// If set to true, the application will print information about running tasks.
-    pub verbose:    bool,
-    /// The command arguments collected as a list of `Token`s
-    pub arguments:  Vec<Token>,
-    /// The inputs supplied that will be used with `arguments`.
-    pub inputs:     Vec<String>
-}
-
 /// The error type for the argument module.
+#[derive(Debug, PartialEq)]
 pub enum ParseErr {
     /// An error occured opening an input file.
     InputFileError(String, String),
@@ -47,6 +25,34 @@ pub enum ParseErr {
     NoArguments
 }
 
+impl ParseErr {
+    pub fn handle(self, stdout: Stdout, stderr: Stderr) -> ! {
+        // Always lock an output buffer before using it.
+        let mut stderr = stderr.lock();
+        let mut stdout = stdout.lock();
+        let _ = stderr.write(b"parallel: parsing error: ");
+        match self {
+            ParseErr::InputFileError(file, why) => {
+                let _ = write!(&mut stderr, "unable to open {}: {}\n", file, why);
+            },
+            ParseErr::JobsNaN(value) => {
+                let _ = write!(&mut stderr, "jobs parameter, '{}', is not a number.\n", value);
+            },
+            ParseErr::JobsNoValue => {
+                let _ = stderr.write(b"no jobs parameter was defined.\n");
+            },
+            ParseErr::InvalidArgument(argument) => {
+                let _ = write!(&mut stderr, "invalid argument: {}\n", argument);
+            },
+            ParseErr::NoArguments => {
+                let _ = write!(&mut stderr, "no input arguments were given.\n");
+            }
+        };
+        let _ = stdout.write(b"For help on command-line usage, execute `parallel -h`\n");
+        exit(1);
+    }
+}
+
 #[derive(PartialEq)]
 enum Mode {
     Arguments,
@@ -55,8 +61,48 @@ enum Mode {
     Files
 }
 
+#[derive(Clone)]
+pub struct Flags {
+    pub grouped:             bool,
+    pub inputs_are_commands: bool,
+    pub uses_shell:          bool,
+    pub quiet:               bool,
+    pub verbose:             bool
+}
+
+impl Flags {
+    fn new() -> Flags {
+        Flags {
+            grouped: true,
+            uses_shell: true,
+            quiet: false,
+            verbose: false,
+            inputs_are_commands: false,
+        }
+    }
+}
+
+/// `Args` is a collection of critical options and arguments that were collected at
+/// startup of the application.
+pub struct Args {
+    pub flags:     Flags,
+    pub ncores:    usize,
+    pub arguments: Vec<Token>,
+    pub ninputs:   usize,
+    pub inputs:    Vec<String>
+}
 
 impl Args {
+    pub fn new() -> Args {
+        Args {
+            ncores: num_cpus::get(),
+            flags: Flags::new(),
+            arguments: Vec::new(),
+            ninputs: 0,
+            inputs: Vec::new()
+        }
+    }
+
     pub fn parse(&mut self) -> Result<(), ParseErr> {
         let mut raw_args = env::args().skip(1).peekable();
         let mut comm = String::with_capacity(128);
@@ -71,7 +117,7 @@ impl Args {
         };
 
         // If there are no arguments to be parsed, then the inputs are commands.
-        self.inputs_are_commands = mode == Mode::Inputs || mode == Mode::Files;
+        self.flags.inputs_are_commands = mode == Mode::Inputs || mode == Mode::Files;
 
         // Parse each and every input argument supplied to the program.
         while let Some(argument) = raw_args.next() {
@@ -96,44 +142,52 @@ impl Args {
                                 } else {
                                     // If there was no character after `j`, the following argument
                                     // must be the job value.
-                                    let ref val = try!(raw_args.next().ok_or(ParseErr::JobsNoValue));
+                                    let val = &try!(raw_args.next().ok_or(ParseErr::JobsNoValue));
                                     try!(jobs::parse(val))
                                 }
                             } else if character != '-' {
-                                // All following characters will be considered their own argument.
-                                let mut char_iter = argument[1..].chars();
-                                while let Some(character) = char_iter.next() {
+                                // NOTE: Short mode versions of arguments
+                                for character in argument[1..].chars() {
                                     match character {
                                         'h' => {
                                             println!("{}", man::MAN_PAGE);
                                             exit(0);
                                         },
-                                        'n' => self.uses_shell = false,
-                                        'u' => self.grouped = false,
-                                        'v' => self.verbose = true,
+                                        'n' => self.flags.uses_shell = false,
+                                        'u' => self.flags.grouped = false,
+                                        'q' => self.flags.quiet = true,
+                                        'v' => self.flags.verbose = true,
                                         _ => {
                                             return Err(ParseErr::InvalidArgument(argument.to_owned()))
                                         }
                                     }
                                 }
                             } else {
-                                // These are all the long mode versions of the arguments.
+                                // NOTE: Long mode versions of arguments
                                 match &argument[2..] {
                                     "help" => {
                                         println!("{}", man::MAN_PAGE);
                                         exit(0);
                                     },
                                     "jobs" => {
-                                        let ref val = try!(raw_args.next().ok_or(ParseErr::JobsNoValue));
+                                        let val = &try!(raw_args.next().ok_or(ParseErr::JobsNoValue));
                                         self.ncores = try!(jobs::parse(val))
                                     },
-                                    "ungroup" => self.grouped = false,
-                                    "no-shell" => self.uses_shell = false,
+                                    "ungroup" => self.flags.grouped = false,
+                                    "no-shell" => self.flags.uses_shell = false,
                                     "num-cpu-cores" => {
                                         println!("{}", num_cpus::get());
                                         exit(0);
                                     },
-                                    "verbose" => self.verbose = true,
+                                    "quiet" => self.flags.quiet = true,
+                                    "verbose" => self.flags.verbose = true,
+                                    "version" => {
+                                        println!("parallel 0.4.2\n\nCrate Dependencies:");
+                                        println!("    libc      0.2.15");
+                                        println!("    num_cpus  1.0.0");
+                                        println!("    permutate 0.1.3");
+                                        exit(0);
+                                    }
                                     _ => {
                                         return Err(ParseErr::InvalidArgument(argument.to_owned()));
                                     }
@@ -147,11 +201,11 @@ impl Args {
                         match argument {
                             ":::" => {
                                 mode = Mode::Inputs;
-                                self.inputs_are_commands = true;
+                                self.flags.inputs_are_commands = true;
                             },
                             "::::" => {
                                 mode = Mode::Files;
-                                self.inputs_are_commands = true;
+                                self.flags.inputs_are_commands = true;
                             }
                             _ => {
                                 // The command has been supplied, and argument parsing is over.
@@ -169,10 +223,11 @@ impl Args {
                     // All other arguments are command arguments.
                     _ => {
                         comm.push(' ');
-                        comm.push_str(&argument);
+                        comm.push_str(argument);
                     }
                 },
                 _ => match argument {
+                    // `:::` denotes that the next set of inputs will be added to a new list.
                     ":::"  => {
                         mode = Mode::Inputs;
                         if !current_inputs.is_empty() {
@@ -180,7 +235,9 @@ impl Args {
                             current_inputs.clear();
                         }
                     },
+                    // `:::+` denotes that the next set of inputs will be added to the current list.
                     ":::+" => mode = Mode::Inputs,
+                    // `::::` denotes that the next set of inputs will be added to a new list.
                     "::::"  => {
                         mode = Mode::Files;
                         if !current_inputs.is_empty() {
@@ -188,7 +245,9 @@ impl Args {
                             current_inputs.clear();
                         }
                     },
+                    // `:::+` denotes that the next set of inputs will be added to the current list.
                     "::::+" => mode = Mode::Files,
+                    // All other arguments will be added to the current list.
                     _ => match mode {
                         Mode::Inputs => current_inputs.push(argument.to_owned()),
                         Mode::Files => try!(file_parse(&mut current_inputs, argument)),
@@ -241,6 +300,8 @@ impl Args {
             }
         }
 
+        self.ninputs = self.inputs.len();
+
         Ok(())
     }
 }
