@@ -17,8 +17,8 @@ pub struct ParallelCommand<'a> {
 }
 
 impl<'a> ParallelCommand<'a> {
-    pub fn exec(&self, grouped: bool, uses_shell: bool, child: &mut Child, jobs: &[String])
-        -> io::Result<CommandResult>
+    pub fn exec(&self, grouped: bool, uses_shell: bool, quiet: bool, child: &mut Child,
+        jobs: &[String]) -> io::Result<CommandResult>
     {
         // First the arguments will be generated based on the tokens and input.
         let mut arguments = String::with_capacity(self.command_template.len() << 1);
@@ -38,9 +38,9 @@ impl<'a> ParallelCommand<'a> {
         }
 
         if grouped {
-            get_command_output(&arguments, uses_shell, child).map(|_| CommandResult::Grouped)
+            get_command_output(&arguments, uses_shell, quiet, child).map(|_| CommandResult::Grouped)
         } else {
-            get_command_status(&arguments, uses_shell).map(|_| CommandResult::Status)
+            get_command_status(&arguments, uses_shell, quiet).map(|_| CommandResult::Status)
         }
     }
 
@@ -85,64 +85,86 @@ impl<'a> ParallelCommand<'a> {
     }
 }
 
-pub fn get_command_output(command: &str, uses_shell: bool, child: &mut Child)
+pub fn get_command_output(command: &str, uses_shell: bool, quiet: bool, child: &mut Child)
     -> io::Result<()>
 {
     if uses_shell {
-        shell_output(command, child)
+        shell_output(command, child, quiet)
     } else {
         let arguments = split_into_args(command);
-        if arguments.len() == 1 {
-            Command::new(&arguments[0])
+        match (arguments.len() == 1, quiet) {
+            (true, true) => Command::new(&arguments[0])
+                .stdout(Stdio::null()).stderr(Stdio::piped())
+                .spawn().map(|process| { *child = process; () }),
+            (true, false) => Command::new(&arguments[0])
                 .stdout(Stdio::piped()).stderr(Stdio::piped())
-                .spawn().map(|process| { *child = process; () })
-        } else {
-            Command::new(&arguments[0]).args(&arguments[1..])
+                .spawn().map(|process| { *child = process; () }),
+            (false, true) => Command::new(&arguments[0]).args(&arguments[1..])
+                .stdout(Stdio::null()).stderr(Stdio::piped())
+                .spawn().map(|process| { *child = process; () }),
+            (false, false) => Command::new(&arguments[0]).args(&arguments[1..])
                 .stdout(Stdio::piped()).stderr(Stdio::piped())
                 .spawn().map(|process| { *child = process; () })
         }
     }
 }
 
-pub fn get_command_status(command: &str, uses_shell: bool) -> io::Result<ExitStatus> {
+pub fn get_command_status(command: &str, uses_shell: bool, quiet: bool) -> io::Result<ExitStatus> {
     if uses_shell {
-        shell_status(command)
+        shell_status(command, quiet)
     } else {
         let arguments = split_into_args(command);
-        if arguments.len() == 1 {
-            Command::new(&arguments[0]).status()
-        } else {
-            Command::new(&arguments[0]).args(&arguments[1..]).status()
+        match (arguments.len() == 1, quiet) {
+            (true, true)   => Command::new(&arguments[0]).stdout(Stdio::null()).status(),
+            (true, false)  => Command::new(&arguments[0]).status(),
+            (false, true)  => Command::new(&arguments[0]).args(&arguments[1..]).stdout(Stdio::null()).status(),
+            (false, false) => Command::new(&arguments[0]).args(&arguments[1..]).status()
         }
     }
 }
 
 #[cfg(windows)]
-fn shell_output<S: AsRef<OsStr>>(args: S, child: &mut Child) -> io::Result<()> {
-    Command::new("cmd").arg("/C").arg(args)
-        .stdout(Stdio::piped())
-        .stderr(Stdio::piped())
-        .spawn().map(|process| {
-            *child = process;
-            ()
-        })
+fn shell_output<S: AsRef<OsStr>>(args: S, child: &mut Child, quiet: bool) -> io::Result<()> {
+    if quiet {
+        Command::new("cmd").arg("/C").arg(args)
+            .stdout(Stdio::null()).stderr(Stdio::piped())
+            .spawn().map(|process| { *child = process; () })
+    } else {
+        Command::new("cmd").arg("/C").arg(args)
+            .stdout(Stdio::piped()).stderr(Stdio::piped())
+            .spawn().map(|process| { *child = process; () })
+    }
 }
 
 #[cfg(windows)]
-fn shell_status<S: AsRef<OsStr>>(args: S) -> io::Result<ExitStatus> {
-    Command::new("cmd").arg("/C").arg(args).status()
+fn shell_status<S: AsRef<OsStr>>(args: S, quiet: bool) -> io::Result<ExitStatus> {
+    if quiet {
+        Command::new("cmd").arg("/C").arg(args).stdout(Stdio::null()).status()
+    } else {
+        Command::new("cmd").arg("/C").arg(args).status()
+    }
 }
 
 #[cfg(not(windows))]
-fn shell_output<S: AsRef<OsStr>>(args: S, child: &mut Child) -> io::Result<()> {
-    Command::new("sh").arg("-c").arg(args)
-        .stdout(Stdio::piped()).stderr(Stdio::piped())
-        .spawn().map(|process| { *child = process; () })
+fn shell_output<S: AsRef<OsStr>>(args: S, child: &mut Child, quiet: bool) -> io::Result<()> {
+    if quiet {
+        Command::new("sh").arg("-c").arg(args)
+            .stdout(Stdio::null()).stderr(Stdio::piped())
+            .spawn().map(|process| { *child = process; () })
+    } else {
+        Command::new("sh").arg("-c").arg(args)
+            .stdout(Stdio::piped()).stderr(Stdio::piped())
+            .spawn().map(|process| { *child = process; () })
+    }
 }
 
 #[cfg(not(windows))]
-fn shell_status<S: AsRef<OsStr>>(args: S) -> io::Result<ExitStatus> {
-    Command::new("sh").arg("-c").arg(args).status()
+fn shell_status<S: AsRef<OsStr>>(args: S, quiet: bool) -> io::Result<ExitStatus> {
+    if quiet {
+        Command::new("sh").arg("-c").arg(args).stdout(Stdio::null()).status()
+    } else {
+        Command::new("sh").arg("-c").arg(args).status()
+    }
 }
 
 /// Handles quoting of arguments to prevent arguments with spaces as being read as
@@ -189,11 +211,9 @@ fn split_into_args(command: &str) -> Vec<String> {
                     if prev_char_was_a_backslash {
                         buffer.push(' ');
                         prev_char_was_a_backslash = false;
-                    } else {
-                        if !buffer.is_empty() {
-                            output.push(buffer.clone());
-                            buffer.clear();
-                        }
+                    } else if !buffer.is_empty() {
+                        output.push(buffer.clone());
+                        buffer.clear();
                     }
                 },
                 '\\' => {
